/* GStreamer Media Player
 * (c) 2004 Ronald Bultje <rbultje@ronald.bitfreak.net>
 *
 * window.c: main window.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <gtk/gtk.h>
#include <gtk/gtkmain.h>
#include <gtk/gtksignal.h>
#include <gdk/gdkx.h>
#include <gst/gconf/gconf.h>
#include <gnome.h>
#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>

#include "stock.h"
#include "timer.h"
#include "video.h"
#include "window.h"

static void	gst_player_window_class_init	(GstPlayerWindowClass *klass);
static void	gst_player_window_init		(GstPlayerWindow *win);
static void	gst_player_window_dispose	(GObject         *object);

static void	cb_open				(GtkWidget       *widget,
						 gpointer         data);
static void	cb_play_or_pause		(GtkWidget       *widget,
						 gpointer         data);
static void	cb_exit				(GtkWidget       *widget,
						 gpointer         data);
static void	cb_about			(GtkWidget       *widget,
						 gpointer         data);

static void	cb_error			(GstPlay         *play,
						 GstElement      *source,
						 GError          *error,
						 gchar           *debug,
						 gpointer         data);
static void	cb_state			(GstPlay         *play,
						 GstElementState  old_state,
						 GstElementState  new_state,
						 gpointer         data);

static GnomeAppClass *parent_class = NULL;

#define GST_PLAYER_ERROR \
  (gst_player_error_quark ())

static GQuark
gst_player_error_quark (void)
{
  static GQuark quark = 0;

  if (quark == 0)
    quark = g_quark_from_static_string ("gst-player-error-quark");

  return quark;
}

GType
gst_player_window_get_type (void)
{
  static GType gst_player_window_type = 0;

  if (!gst_player_window_type) {
    static const GTypeInfo gst_player_window_info = {
      sizeof (GstPlayerWindowClass),
      NULL,
      NULL,
      (GClassInitFunc) gst_player_window_class_init,
      NULL,
      NULL,
      sizeof (GstPlayerWindow),
      0,
      (GInstanceInitFunc) gst_player_window_init,
      NULL
    };

    gst_player_window_type =
	g_type_register_static (GNOME_TYPE_APP, 
				"GstPlayerWindow",
				&gst_player_window_info, 0);
  }

  return gst_player_window_type;
}

static void
gst_player_window_class_init (GstPlayerWindowClass *klass)
{
  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);

  parent_class = g_type_class_ref (GNOME_TYPE_APP);

  gobject_class->dispose = gst_player_window_dispose;
}

/*
 * Menus.
 */

#define GNOMEUIINFO_ITEM_QUICKKEY(label, tip, cb, stock_id, quickkey) \
	{ GNOME_APP_UI_ITEM, label, tip, (gpointer) cb, \
	  NULL, NULL, GNOME_APP_PIXMAP_STOCK, stock_id, \
	  quickkey, GDK_CONTROL_MASK, NULL }

static GnomeUIInfo file_menu[] = {
  GNOMEUIINFO_MENU_OPEN_ITEM (cb_open, NULL),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_ITEM_QUICKKEY (N_("_Play / Pause"), N_("Play or pause"),
			     cb_play_or_pause, GST_PLAYER_STOCK_PLAY, 'p'),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_MENU_EXIT_ITEM (cb_exit, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo edit_menu[] = {
  /* preferences */
  GNOMEUIINFO_END
};

static GnomeUIInfo view_menu[] = {
  /* ? */
  GNOMEUIINFO_END
};

static GnomeUIInfo help_menu[] = {
  GNOMEUIINFO_HELP (PACKAGE),
  GNOMEUIINFO_MENU_ABOUT_ITEM (cb_about, NULL),
  GNOMEUIINFO_END
};

static GnomeUIInfo menu[] = {
  GNOMEUIINFO_MENU_FILE_TREE (file_menu),
  GNOMEUIINFO_MENU_EDIT_TREE (edit_menu),
  GNOMEUIINFO_MENU_VIEW_TREE (view_menu),
  GNOMEUIINFO_MENU_HELP_TREE (help_menu),
  GNOMEUIINFO_END
};

/*
 * Toolbar.
 */
                                                                                
static GnomeUIInfo tool[] = {
  GNOMEUIINFO_ITEM_STOCK (N_("Play"), N_("Start playing"),
			  cb_play_or_pause, GST_PLAYER_STOCK_PLAY),
  GNOMEUIINFO_ITEM_STOCK (N_("Pause"), N_("Pause playback"),
			  cb_play_or_pause, GST_PLAYER_STOCK_PAUSE),
  GNOMEUIINFO_SEPARATOR,
  GNOMEUIINFO_END
};

static void
gst_player_window_init (GstPlayerWindow *win)
{
  GnomeApp *app = GNOME_APP (win);
  GtkWidget *bar;

  win->play = NULL;

  /* init */
  gnome_app_construct (app, PACKAGE, PACKAGE_NAME);

  /* menus, toolbar */
  gnome_app_create_menus_with_data (app, menu, win);
  gnome_app_create_toolbar_with_data (app, tool, win);
  gtk_widget_hide (tool[1].widget);

  /* statusbar */
  bar = gnome_appbar_new (FALSE, TRUE, GNOME_PREFERENCES_USER);
  gnome_app_set_statusbar (app, bar);
  gnome_app_install_appbar_menu_hints (GNOME_APPBAR (bar), menu);
}

GtkWidget *
gst_player_window_new (GError **err)
{
  GstPlayerWindow *win;
  GstPlay *play;
  GstElement *data, *audio, *video;
  GnomeApp *app;
  GtkWidget *videow, *toolbar, *slider;
  BonoboDockItem *item;

  /* start with the player */
  if (!(play = gst_play_new (GST_PLAY_MODE_VIDEO, err)))
    return NULL;

  /* set data input and video/audio output */
  if (!(data = gst_element_factory_make ("gnomevfssrc", "source"))) {
    g_set_error (err, GST_PLAYER_ERROR, 1,
		 _("Failed to create gnomevfssrc element"));
    goto fail;
  }
  if (!gst_play_set_data_input (play, data, err))
    goto fail;

  if (!(audio = gst_gconf_get_default_audio_sink ())) {
    g_set_error (err, GST_PLAYER_ERROR, 1,
		 _("Failed to obtain default audio sink from GConf"));
    goto fail;
  }
  if (!gst_play_set_audio_output (play, audio, err))
    goto fail;

  if (!(video = gst_gconf_get_default_video_sink ())) {
    g_set_error (err, GST_PLAYER_ERROR, 1,
		 _("Failed to obtain default video sink from GConf"));
    goto fail;
  }
  if (!gst_play_set_video_output (play, video, err))
    goto fail;

  if (gst_element_set_state (GST_ELEMENT (play),
			     GST_STATE_READY) != GST_STATE_SUCCESS) {
    g_set_error (err, GST_PLAYER_ERROR, 1,
		 _("Failed to set player to initial ready state - fatal"));
    goto fail;
  }

  /* actual window */
  win = g_object_new (GST_PLAYER_TYPE_WINDOW, NULL);
  app = GNOME_APP (win);
  win->play = play;
  g_signal_connect (play, "error", G_CALLBACK (cb_error), win);
  g_signal_connect (play, "state-change", G_CALLBACK (cb_state), win);

  /* add slider */
  slider = gst_player_timer_new (play);
  item = gnome_app_get_dock_item_by_name (app, GNOME_APP_TOOLBAR_NAME);
  toolbar = bonobo_dock_item_get_child (item);
  gtk_toolbar_append_widget (GTK_TOOLBAR (toolbar), slider,
                             _("Time"), NULL);
  gtk_widget_show (slider);

  /* video widget */
  videow = gst_player_video_new (video);
  gnome_app_set_contents (app, videow);
  gtk_widget_show (videow);

  return GTK_WIDGET (win);

fail:
  gst_object_unref (GST_OBJECT (play));
  return NULL;
}

static void
gst_player_window_dispose (GObject *object)
{
  GstPlayerWindow *win = GST_PLAYER_WINDOW (object);

  if (win->play) {
    gst_element_set_state (GST_ELEMENT (win->play), GST_STATE_NULL);
    gst_object_unref (GST_OBJECT (win->play));
    win->play = NULL;
  }

  G_OBJECT_CLASS (parent_class)->dispose (object);
}

/*
 * Menu/Toolbar actions.
 */

static void
cb_open (GtkWidget *widget,
	 gpointer   data)
{
  GstPlayerWindow *win = GST_PLAYER_WINDOW (data);
  GstElement *pel = GST_ELEMENT (win->play);
  GtkWidget *filesel;
  const gchar *location;

  filesel = gtk_file_selection_new (_("Open media file"));
  gtk_widget_show (filesel);
  if (gtk_dialog_run (GTK_DIALOG (filesel)) != GTK_RESPONSE_ACCEPT) {
    location = gtk_file_selection_get_filename (GTK_FILE_SELECTION (filesel));
    gtk_widget_destroy (filesel);

    gst_play_set_location (win->play, location);
    gst_element_set_state (pel, GST_STATE_PLAYING);
  } else {
    gtk_widget_destroy (filesel);
  }
}

static void
cb_play_or_pause (GtkWidget *widget,
		  gpointer   data)
{
  GstPlayerWindow *win = GST_PLAYER_WINDOW (data);
  GstElementState state;

  if (GST_STATE (win->play) == GST_STATE_PLAYING)
    state = GST_STATE_PAUSED;
  else
    state = GST_STATE_PLAYING;

  gst_element_set_state (GST_ELEMENT (win->play), state);
}

static void
cb_exit (GtkWidget *widget,
	 gpointer   data)
{
  gtk_widget_destroy (GTK_WIDGET (data));
}

static void
cb_about (GtkWidget *widget,
	  gpointer   data)
{
  GtkWidget *about;
  const gchar *authors[] = { "Ronald Bultje <rbultje@ronald.bitfreak.net>",
                             NULL };

  about = gnome_about_new (_(PACKAGE_NAME), VERSION,
                           "(c) 2004 Ronald Bultje",
                           _("A GNOME/GStreamer-based media player"),
                           authors, NULL, NULL, NULL);

  gtk_widget_show (about);
}

typedef struct _GstPlayerWindowError {
  GstPlay *play;
  GstElement *source;
  GError *error;
  gchar *debug;
  GstPlayerWindow *win;
} GstPlayerWindowError;

static gboolean
idle_error (gpointer data)
{
  GstPlayerWindowError *err = data;
  GtkWidget *dialog;

  dialog = gtk_message_dialog_new (GTK_WINDOW (err->win),
                                   GTK_DIALOG_DESTROY_WITH_PARENT,
                                   GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
                                   err->error->message);
  gtk_widget_show (dialog);
  gtk_dialog_run (GTK_DIALOG (dialog));
  gtk_widget_destroy (dialog);

  g_free (err->debug);
  g_error_free (err->error);
  gst_object_unref (GST_OBJECT (err->play));
  gst_object_unref (GST_OBJECT (err->source));
  g_object_unref (G_OBJECT (err->win));
  g_free (err);

  /* once */
  return FALSE;
}

static void
cb_error (GstPlay    *play,
	  GstElement *source,
	  GError     *error,
	  gchar      *debug,
	  gpointer    data)
{
  GstPlayerWindow *win = GST_PLAYER_WINDOW (data);
  GstPlayerWindowError *err = g_new (GstPlayerWindowError, 1);

  err->play = play;
  gst_object_ref (GST_OBJECT (play));
  err->source = source;
  gst_object_ref (GST_OBJECT (source));
  err->error = g_error_copy (error);
  err->debug = g_strdup (debug);
  err->win = win;
  g_object_ref (G_OBJECT (win));

  g_idle_add ((GSourceFunc) idle_error, err);
}

typedef struct _GstPlayerWindowStateChange {
  GstPlay *play;
  GstElementState old_state, new_state;
  GstPlayerWindow *win;
} GstPlayerWindowStateChange;

static gboolean
idle_state (gpointer data)
{
  GstPlayerWindowStateChange *st = data;

  if (st->old_state == GST_STATE_PLAYING) {
    /* show play button */
    gtk_widget_show (tool[0].widget);
    gtk_widget_hide (tool[1].widget);
  } else if (st->new_state == GST_STATE_PLAYING) {
    /* show pause button */
    gtk_widget_show (tool[1].widget);
    gtk_widget_hide (tool[0].widget);
  }

  gst_object_unref (GST_OBJECT (st->play));
  g_object_unref (G_OBJECT (st->win));
  g_free (st);

  /* once */
  return FALSE;
}

static void
cb_state (GstPlay        *play,
	  GstElementState old_state,
	  GstElementState new_state,
	  gpointer        data)
{
  GstPlayerWindow *win = GST_PLAYER_WINDOW (data);
  GstPlayerWindowStateChange *st = g_new (GstPlayerWindowStateChange, 1);

  st->play = play;
  gst_object_ref (GST_OBJECT (play));
  st->old_state = old_state;
  st->new_state = new_state;
  st->win = win;
  g_object_ref (G_OBJECT (win));

  g_idle_add ((GSourceFunc) idle_state, st);
}
